/*******************************************************************************
*                                                                              *
* System      : SOIPIX readout module                                          *
* Block       : Example for the User FPGA                                      *
* Module      : SOIPIX_USER                                                    *
* Version     : v 0.0.0 2008/05/19                                             *
*                                                                              *
* Description : Top Module                                                     *
*                                                                              *
* Designer    : Tomohisa Uchida                                                *
*                                                                              *
*                Copyright (c) 2008 Tomohisa Uchida                            *
*                All rights reserved                                           *
*                                                                              *
*******************************************************************************/
`define FPGA_VER 32'h0805_1901
`define FPGA_ID  32'h0005_0000

`include "../Src/sys_dcm.v"
`include "../Src/SIO_SLAVE.v"
`include "../Src/SIO_REG.v"

module SOIPIX_USER(
// System
	OSC					,	// in	: 50MHz clock
	SW_RSTn				,	// in	: System reset
// User I/F
	USR_CLK				,	// out	: Clock
	USR_ACTIVE			,	// in	: TCP established
	USR_CLOSE_REQ		,	// in	: Request to close a TCP Connection
	USR_CLOSE_ACK		,	// out	: Acknowledge for USR_CLOSE_REQ
	// TCP Tx
	USR_TX_AFULL		,	// in	: Almost full flag of a TCP Tx FIFO
	USR_TX_WE			,	// out	: TCP Tx Data write enable
	USR_TX_WD			,	// out	: TCP Tx Data[7:0]
	// TCP Rx
	USR_RX_EMPTY		,	// in	: Empty flag of a TCP RX flag
	USR_RX_RE			,	// out	: TCP Rx Data Read enable
	USR_RX_RV			,	// in	: TCP Rx data valid
	USR_RX_RD			,	// in	: TCP Rx data[7:0]
	// Register Access
	REG_CLK				,	// in	: Clock
	REG_ACT				,	// in	: Access Active
	REG_DI				,	// out	: Serial Data Output
	REG_DO				,	// in	: Serial Data Input
// SW, LED
	LED					,	// out	: LED[7:0]
	DIP_SW					// in	: DIP SW[7:0]
);

//-------- Input/Output -------------
	input			OSC				;
	input			SW_RSTn			;

	output			USR_CLK			;
	input			USR_ACTIVE		;
	input			USR_CLOSE_REQ	;
	output			USR_CLOSE_ACK	;
	input			USR_TX_AFULL	;
	output			USR_TX_WE		;
	output	[7:0]	USR_TX_WD		;
	input			USR_RX_EMPTY	;
	output			USR_RX_RE		;
	input			USR_RX_RV		;
	input	[7:0]	USR_RX_RD		;
	input			REG_CLK			;
	input			REG_ACT			;
	output			REG_DI			;
	input			REG_DO			;

	output	[7:0]	LED				;
	input	[7:0]	DIP_SW			;

//------------------------------------------------------------------------------
//	DLL
//------------------------------------------------------------------------------
	wire			RST_DLL				;
	wire			locked				;
	wire			CLK25M				;
	wire			CLK50M				;

	assign	RST_DLL		= ~SW_RSTn;

   	sys_dcm			SYSDCM(
		.CLKIN_IN			(OSC			),
		.RST_IN				(RST_DLL		),
		.CLKDV_OUT			(CLK25M			),
		.CLKIN_IBUFG_OUT	(),
		.CLK0_OUT			(CLK50M			),
		.LOCKED_OUT			(locked			)
	);

	assign	RSTn	= locked & ~RST_DLL;

	ODDR	IOB_DDRCLKp0(.Q(USR_CLK), .C(CLK25M), .CE(1'b1), .D1(1'b0), .D2(1'b1), .R( 1'b0), .S( 1'b0));

//------------------------------------------------------------------------------
//	LED
//------------------------------------------------------------------------------
	wire	[7:0]	LED				;

	reg		[23:0]	ledCnt			;
	reg		[7:0]	ledOut			;

	always@ (posedge CLK25M or negedge RSTn) begin
		if(~RSTn)begin
			ledCnt[23:0] <= 24'd0;
			ledOut[7:0]	<= 8'd1;
		end else begin
			ledCnt[23:0] <= ledCnt[23:0] + 24'd1;
			if(ledCnt[23:0]==24'hFF_FFFF)begin
				ledOut[7:0]	<= {ledOut[6:0],ledOut[7]};
			end
		end
	end

	assign	LED[0] = DIP_SW[0] | ledOut[0];
	assign	LED[1] = DIP_SW[1] | ledOut[1];
	assign	LED[2] = DIP_SW[2] | ledOut[2];
	assign	LED[3] = DIP_SW[3] | ledOut[3];
	assign	LED[4] = DIP_SW[4] | ledOut[4];
	assign	LED[5] = DIP_SW[5] | ledOut[5];
	assign	LED[6] = DIP_SW[6] | ledOut[6];
	assign	LED[7] = DIP_SW[7] | ledOut[7];

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
	reg				irUsrActive		;
	reg				irUsrCloseReq	;
	reg				irUsrTxAfull	;
	reg				irUsrRxEmpty	;
	reg				irUsrRxRv		;
	reg		[7:0]	irUsrRxRd		;

	always@ (posedge CLK25M) begin
		irUsrActive		<= USR_ACTIVE;
		irUsrCloseReq	<= USR_CLOSE_REQ;
		irUsrTxAfull	<= USR_TX_AFULL;
		irUsrRxEmpty	<= USR_RX_EMPTY;
		irUsrRxRv		<= USR_RX_RV;
		irUsrRxRd[7:0]	<= USR_RX_RD[7:0];
	end

	reg				usrTxWe		;
	reg		[7:0]	usrTxWd		;
	reg				usrRxRe		;

	always@ (posedge CLK25M) begin
		usrTxWe		<=  irUsrActive & ~irUsrCloseReq & ~irUsrTxAfull & irUsrRxRv;
		usrTxWd[7:0]	<=  irUsrRxRd[7:0];
		usrRxRe		<=  irUsrActive & ~irUsrCloseReq & ~irUsrRxEmpty;
	end

	reg				orUsrCloseAck	;
	reg				orUsrTxWe		;
	reg		[7:0]	orUsrTxWd		;
	reg				orUsrRxRe		;

	always@ (posedge CLK25M) begin
		orUsrCloseAck	<= irUsrCloseReq;
		orUsrTxWe		<= usrTxWe;
		orUsrTxWd[7:0]	<= usrTxWd[7:0];
		orUsrRxRe		<= usrRxRe;
	end

	assign	USR_CLOSE_ACK	= orUsrCloseAck;
	assign	USR_TX_WE		= orUsrTxWe;
	assign	USR_TX_WD		= orUsrTxWd[7:0];
	assign	USR_RX_RE		= orUsrRxRe;

//------------------------------------------------------------------------------
//	Register Access
//------------------------------------------------------------------------------
	wire	[31:0]	SIO_ADDR		;
	wire	[7:0]	SIO_WD			;
	wire			SIO_WE			;
	wire			SIO_RE			;
	wire			SIO_ACK			;
	wire			SIO_RV			;
	wire	[7:0]	SIO_RD			;

	SIO_SLAVE			SIO_SLAVE(
		.RSTn					(RSTn					),	// in	: System reset
		.FILL_ADDR				(32'h0					),	// in	: Filled address for narow address-width
		// Serial I/F
		.SCK					(REG_CLK				),	// in	: Clock
		.SCS					(REG_ACT				),	// in	: Active
		.SI						(REG_DI					),	// out	: Data input
		.SO						(REG_DO					),	// in	: Data output
		// Register I/F
		.REG_ADDR				(SIO_ADDR[31:0]			),	// out	: Address[31:0]
		.REG_WD					(SIO_WD[7:0]			),	// out	: Data[7:0]
		.REG_WE					(SIO_WE					),	// out	: Write enable
		.REG_RE					(SIO_RE					),	// out	: Read enable
		.REG_ACK				(SIO_ACK				),	// in	: Access acknowledge
		.REG_RV					(SIO_RV					),	// in	: Read valid
		.REG_RD					(SIO_RD[7:0]			)	// in	: Read data[7:0]
	);

	SIO_REG				SIO_REG(
		.SCK					(REG_CLK				),	// in	: Clock
		.RSTn					(RSTn					),	// in	: System reset
		// Register I/F	
		.REG_ADDR				(SIO_ADDR[31:0]			),	// in	: Address[31:0]
		.REG_WD					(SIO_WD[7:0]			),	// in	: Data[7:0]
		.REG_WE					(SIO_WE					),	// in	: Write enable
		.REG_RE					(SIO_RE					),	// in	: Read enable
		.REG_ACK				(SIO_ACK				),	// out	: Access acknowledge
		.REG_RV					(SIO_RV					),	// out	: Read valid
		.REG_RD					(SIO_RD[7:0]			)	// out	: Read data[7:0]
	);

//------------------------------------------------------------------------------
endmodule